package controller;

import java.io.File;

import model.StudentModel;
import model.beans.Student;
import model.beans.factories.StudentFactory;
import model.comparators.Condition;
import model.comparators.factories.StudentComparatorFactory;
import saver.Saver;
import view.AddStudentView;
import view.LoadView;
import view.MainView;
import view.SaveView;
import view.SearchStudentView;

public class AppStudentController implements MainStudentController, AddStudentController, SearchStudentController, SaveController, LoadController, Controller {
	StudentModel model;
	StudentModel searchModel;
	@Override
	public void runAddDialog() {
		new AddStudentView(this).show();
		
	}

	@Override
	public void runSaveDialog() {
		new SaveView(this).show();
		
	}

	@Override
	public void runSearchDialog() {
		new SearchStudentView(this, searchModel.getTableModel()).show();
		
	}

	@Override
	public void start() {
		initModels();
		initView();
	}

	private void initView() {
		new MainView(this, model.getTableModel()).show();
	}

	private void initModels() {
		model = new StudentModel();
		searchModel = new StudentModel();
	}

	@Override
	public void add(String name, String surname, String patronymic, String sport, String position, String category, String memberInTeam, String numberOfTitels) throws Exception {
		Student student = StudentFactory.newInstance(name, surname, patronymic, sport, position, category, memberInTeam, numberOfTitels);
		if (student.isEmpty()) throw new Exception();
		model.add(student);
	}

	@Override
	public void search(String name, String surname, String patronymic, String sport, String position, String category, String memberInTeam, String numberOfTitels, Condition condition) throws Exception {
		searchModel.clear();
		Student student = StudentFactory.newInstance(name, surname, patronymic, sport, position, category, memberInTeam, numberOfTitels);
		searchModel.add(model.search(student, StudentComparatorFactory.newInstance(condition)));
	}

	@Override
	public void delete() {
		model.delete(searchModel.getStudents());
		searchModel.clear();
	}

	@Override
	public void save(File file) {
		new Saver().save(model, file);
	}

	@Override
	public void runLoadDialog() {
		new LoadView(this).show();
	}

	@Override
	public void load(File file) {
		new Saver().load(model, file);
		
	}

}
