package model;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;

import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableModel;

import model.beans.Student;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import saver.Savable;

public class StudentModel implements Savable {
	private List<Student> students;
	private DefaultTableModel model;
	public StudentModel() {
		students = new ArrayList<Student>();
		model = new DefaultTableModel(new Object[]{"Full name", "Sport", "Position", "Category", "Is a member", "Number of titels"}, 0);
	}
	public void add(Student student) {
		students.add(student);
		updateModel();
	}

	public TableModel getTableModel() {
		return model;
	}

	public void add(List<Student> beans) {
		for(Student student: beans) add(student);
	}

	public void clear() {
		students.clear();
		updateModel();
	}
	
	private void updateModel() {
		while(model.getRowCount() != 0) 
			model.removeRow(0);
		for(Student student: students)
			model.addRow(new Object[]{student.getFullName(), student.getSport(), student.getPosition(), student.getCategory(), student.getMemberInTeam(), student.getNumberOfTitels()});
	}
	
	public List<Student> search(Student student, Comparator<Student> comparator) {
		List<Student> searchResults = new ArrayList<>();
		for(Student s: students) {
			if (comparator.compare(student, s) == 0) 
				searchResults.add(s);
		}
		return searchResults;
	}
	
	public void delete(List<Student> students) {
		this.students.removeAll(students);
		updateModel();
	}
	public List<Student> getStudents() {
		return students;
	}
	@Override
	public Element save(Document document) throws Exception {
		Element model = document.createElement("model");
		for(Student s: students)
			if (s != null)
			model.appendChild(s.save(document));
		return model;
	}
	
	@Override
	public void load(Node node) throws Exception {
		students.clear();
		if (node.getNodeName() == "model") {
			NodeList allNodes = node.getChildNodes(); 
			for(int i = 0; i < allNodes.getLength(); i++) {
				Student s = new Student();
				s.load(allNodes.item(i));
				students.add(s);
			}
		}
		updateModel();
	}
	
}
