package view;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.table.TableModel;

import model.comparators.Condition;
import view.components.JPageTable;
import view.components.JStudentAdder;
import controller.SearchStudentController;

public class SearchStudentView implements View {
	private SearchStudentController controller;
	private JFrame frame;
	private JStudentAdder studentAdder;
	private JButton deleteButton;
	public SearchStudentView(SearchStudentController controller, TableModel tableModel) {
		this.controller = controller;
		initFrame(tableModel);
	}
	private void initFrame(TableModel tableModel) {
		frame = new JFrame("Search/Delete...");
		frame.setLayout(new BorderLayout());
		frame.setSize(1000, 400);
		frame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		frame.setLocationRelativeTo(null);
		initElements(tableModel);
	}
	
	private void initElements(TableModel tableModel) {
		initStudentAdder();
		initTable(tableModel);
		initButtons();
	}
	private void initButtons() {
		JPanel panel = new JPanel();
		JButton okButton = new JButton("");
		okButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				try {
					controller.search(studentAdder.getName(),
							studentAdder.getSurname(),
							studentAdder.getPatronymic(),
							studentAdder.getSport(), studentAdder.getPosition(),
							studentAdder.getCategory(),
							studentAdder.getIsAMember(),
							studentAdder.getNumberOfTitels(),
							Condition.EQUAL);
					deleteButton.setEnabled(true);
				} catch (Exception exception) {
					JOptionPane.showMessageDialog(frame, "Error!");
				}
			}
		});
		panel.add(okButton);
		deleteButton = new JButton("");
		deleteButton.setEnabled(false);
		deleteButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				try {
					controller.delete();
					deleteButton.setEnabled(false);
				}
				catch (Exception exception) {
					JOptionPane.showMessageDialog(frame, "Error!");
					exception.printStackTrace();
				}
			}
		});
		panel.add(deleteButton);
		frame.add(panel, BorderLayout.SOUTH);
	}
	private void initStudentAdder() {
		studentAdder = new JStudentAdder();
		frame.add(studentAdder, BorderLayout.CENTER);
		
	}
	private void initTable(TableModel tableModel) {
		frame.add(new JPageTable(tableModel), BorderLayout.EAST);
		
	}
	@Override
	public void show() {
		frame.setVisible(true);
	}

}
