package view.components;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableModel;

public class JPageTable extends JPanel {
	private static final long serialVersionUID = 6945483489335526470L;
	private TableModel fullTable;
	private DefaultTableModel pageTable;
	private int currentPage;
	private int rowCountOnPage;
	private JLabel currentPageLabel;
	private JLabel pageCountLabel;
	private JLabel rowCountOnPageLabel;
	private JLabel rowCountLabel;
	private JTable table;
	private JScrollPane pane;

	public JPageTable(TableModel model) {
		this.setLayout(new BorderLayout());
		currentPage = 0;
		rowCountOnPage = 1;
		currentPageLabel = new JLabel();
		pageCountLabel = new JLabel();
		rowCountOnPageLabel = new JLabel();
		rowCountLabel = new JLabel();
		fullTable = model;
		initPageTable();
		fullTable.addTableModelListener(new TableModelListener() {
			@Override
			public void tableChanged(TableModelEvent e) {
				update();
			}
		});

		JButton nextButton = new JButton("");
		nextButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				toNextPage();
			}
		});
		JButton prevButton = new JButton("");
		prevButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				toPreviousPage();
			}
		});
		JButton firstButton = new JButton("");
		firstButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				toFirstPage();
			}
		});
		JButton lastButton = new JButton("");
		lastButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				toLastPage();
			}
		});
		JButton increaseRowsButton = new JButton("+");
		increaseRowsButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				increaseRowCountOnPage();
			}
		});
		JButton decreaseRowsButton = new JButton("-");
		decreaseRowsButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				decreaseRowCountOnPage();
			}
		});

		JPanel buttons = new JPanel();
		JPanel info = new JPanel();
		buttons.add(firstButton);
		buttons.add(prevButton);
		buttons.add(nextButton);
		buttons.add(lastButton);
		info.add(currentPageLabel);
		info.add(new JLabel(" / "));
		info.add(pageCountLabel);
		info.add(new JLabel("On page:"));
		info.add(rowCountOnPageLabel);
		//info.add(decreaseRowsButton);
		//info.add(increaseRowsButton);
		info.add(new JLabel("  :"));
		info.add(rowCountLabel);

		table = new JTable(pageTable);
		System.out.println(table.getHeight()/table.getRowHeight());
		this.addComponentListener(new ComponentListener() {
			
			@Override
			public void componentShown(ComponentEvent e) {
				// TODO Auto-generated method stub
				
			}
			
			@Override
			public void componentResized(ComponentEvent e) {
				updateRowCountOnPage();

				
			}
			


			@Override
			public void componentMoved(ComponentEvent e) {
				// TODO Auto-generated method stub
				
			}
			
			@Override
			public void componentHidden(ComponentEvent e) {
				// TODO Auto-generated method stub
				
			}
		});
		table.setEnabled(false);
		pane = new JScrollPane(table);
		this.add(pane, BorderLayout.CENTER);
		JPanel us = new JPanel();
		us.add(buttons);
		us.add(info);
		this.add(us, BorderLayout.SOUTH);
		update();
	}

	private void initPageTable() {
		String[] columns = new String[fullTable.getColumnCount()];
		for (int i = 0; i < fullTable.getColumnCount(); i++)
			columns[i] = fullTable.getColumnName(i);
		pageTable = new DefaultTableModel(null, columns);

	}

	private void update() {
		clear();
		for (int i = getStartOfBlock(); i < getFinishOfBlock(); i++)
			pageTable.addRow(getRowAt(i));
		currentPageLabel.setText((currentPage + 1) + "");
		pageCountLabel.setText(getPageCount() + "");
		rowCountOnPageLabel.setText(rowCountOnPage + "");
		rowCountLabel.setText(fullTable.getRowCount() + "");
		updateRowCountOnPage();
	}

	private Object[] getRowAt(int row) {
		Object[] objects = new Object[fullTable.getColumnCount()];
		for (int i = 0; i < fullTable.getColumnCount(); i++)
			objects[i] = fullTable.getValueAt(row, i);
		return objects;
	}

	private void clear() {
		int rowCount = pageTable.getRowCount();
		for (int i = rowCount - 1; i >= 0; i--) {
			pageTable.removeRow(i);
		}
	}

	public void toNextPage() {
		if (currentPage < getPageCount() - 1)
			currentPage++;
		update();
	}

	public void toPreviousPage() {
		if (currentPage > 0)
			currentPage--;
		update();
	}

	public void toFirstPage() {
		currentPage = 0;
		update();
	}

	public void toLastPage() {
		currentPage = Math.max(getPageCount() - 1, 0);
		update();
	}

	public void decreaseRowCountOnPage() {
		if (rowCountOnPage > 1) {
			this.rowCountOnPage--;
			currentPage = 0;
			update();
		}
	}

	public void increaseRowCountOnPage() {
		if (rowCountOnPage < fullTable.getRowCount()) {
			this.rowCountOnPage++;
			currentPage = 0;
			update();
		}
	}

	public int getRowCountOnPage() {
		return rowCountOnPage;
	}

	public int getCurrentPage() {
		return currentPage;
	}

	public int getPageCount() {
		int page = fullTable.getRowCount() / rowCountOnPage;
		if (fullTable.getRowCount() % rowCountOnPage != 0)
			page++;
		return page;
	}

	private int getStartOfBlock() {
		return currentPage * rowCountOnPage;
	}

	private int getFinishOfBlock() {
		return Math.min(getStartOfBlock() + rowCountOnPage,
				fullTable.getRowCount());
	}
	private void updateRowCountOnPage() {
		rowCountOnPage=pane.getHeight()/table.getRowHeight()+1;
		System.out.println(pane.getHeight());
		System.out.println(table.getRowHeight());
		//System.out.println(pane.getHeight()/table.getRowHeight()+1);
		
	}
}
