package Graph;

import java.awt.BasicStroke;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.MouseInfo;
import java.awt.Point;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionListener;
import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseWheelListener;
import java.math.BigDecimal;
import java.util.ArrayList;

import javax.swing.JPanel;



public class GraphPanel extends JPanel implements GraphModelListener {
    private double factorOfShiftX;
    private double factorOfShiftY;
    private double valueOfDivisionX;
    private double valueOfDivisionY;
    private double frequencyDivisionX;
    private double frequencyDivisionY;
    private String nameAxeX = "X";
    private String nameAxeY = "Y";
    GraphModel model;
    ArrayList<double[]> list = null;

    public GraphPanel() {
	factorOfShiftX = 0.5;
	factorOfShiftY = 0.5;
	valueOfDivisionX = 60;
	valueOfDivisionY = 60;
	list = new ArrayList<>();
	addMouseWheelListener(new MouseWheelListener() {
	    @Override
	    public void mouseWheelMoved(MouseWheelEvent e) {
		for (int i = 0; i < e.getWheelRotation(); i++) {
		    increaseScale();
		}
		for (int i = 0; i > e.getWheelRotation(); i--) {
		    reducedScale();
		}
	    }
	});

	this.addMouseMotionListener(new MouseMotionListener() {
	    double a, b, c, d;

	    @Override
	    public void mouseDragged(MouseEvent e) {
		Point location = MouseInfo.getPointerInfo().getLocation();
		c = location.getX();
		d = location.getY();
		moveDrag((int) c - a, (int) d - b);
		a = location.getX();
		b = location.getY();
	    }

	    @Override
	    public void mouseMoved(MouseEvent e) {
		Point location = MouseInfo.getPointerInfo().getLocation();
		a = location.getX();
		b = location.getY();
	    }
	});

    }
    
    public void setList(ArrayList<double[]> list){
	this.list=list;
	repaint();
    }

    public GraphPanel(GraphModel model) {
	this();
	this.model = model;
	try {
	    ((GraphModelEventer) model).addGraphModelListener(this);
	} catch (Exception e) {
	    // TODO: handle exception
	}

	if (model.getGraph() != null) {
	    list = model.getGraph();
	}
	list = new ArrayList<>();
	nameAxeX = model.getNameAxsX();
	nameAxeY = model.getNameAxsY();
    }

    @Override
    public void paint(Graphics g) {
	super.paint(g);
	Graphics2D g2 = (Graphics2D) g;
	g2.setStroke(new BasicStroke(getLineWidth()));
	drawAxis(g2);
	signAxes(g2);
	paintDivision(g2);
	buildSchedule(g2);

    }

    /*
     * public void draw(ArrayList<double[]> list) { this.list = list;
     * this.repaint(); }
     */

    public double getFactorOfShiftY() {
	return factorOfShiftY;
    }

    public void setFactorOfShiftY(double factorOfShiftY) {
	this.factorOfShiftY = factorOfShiftY;
	this.repaint();
    }

    public double getFactorOfShiftX() {
	return factorOfShiftX;

    }

    public void setFactorOfShiftX(double factorOfShiftX) {
	this.factorOfShiftX = factorOfShiftX;
	this.repaint();
    }

    public int getShiftX() {
	return (int) (getFactorOfShiftX() * getSize().getWidth());
    }

    public int getShiftY() {
	return (int) (getFactorOfShiftY() * getSize().getHeight());
    }

    public int getLineWidth() {
	return (int) (getSize().getWidth() / 200) < (int) (getSize()
		.getHeight() / 200) ? (int) (getSize().getWidth() / 200)
		: (int) (getSize().getHeight() / 200);
    }

    public int getRatio() {
	return (int) (getSize().getWidth() / 80) < (int) (getSize().getHeight() / 80) ? (int) (getSize()
		.getWidth() / 80) : (int) (getSize().getHeight() / 80);
    }

    public double getValueOfDivisionX() {
	return valueOfDivisionX;
    }

    public void setValueOfDivisionX(double valueOfDivisionX) {
	this.valueOfDivisionX = valueOfDivisionX;
	this.repaint();
    }

    public double getValueOfDivisionY() {
	return valueOfDivisionY;
    }

    public void setValueOfDivisionY(double valueOfDivisionY) {
	this.valueOfDivisionY = valueOfDivisionY;
	this.repaint();
    }

    void increaseScale() {
	if ((getValueOfDivisionX() > 0.01) && (getValueOfDivisionY() > 0.01)) {
	    setValueOfDivisionX((getValueOfDivisionX() / 2));
	    setValueOfDivisionY((getValueOfDivisionY() / 2));
	}
    }

    void reducedScale() {
	if ((getValueOfDivisionX() < 1000) && (getValueOfDivisionY() < 1000)) {
	    setValueOfDivisionX((getValueOfDivisionX() * 2));
	    setValueOfDivisionY((getValueOfDivisionY() * 2));
	}
    }

    void moveDrag(double x, double y) {
	setFactorOfShiftY(getFactorOfShiftY() + y / 800);
	setFactorOfShiftX(getFactorOfShiftX() + x / 800);

    }

    void drawAxis(Graphics g) {
	g.drawLine(getShiftX(), 0, getShiftX(), (int) getSize().getHeight());
	g.drawLine(0, getShiftY(), (int) getSize().getWidth(), getShiftY());
	g.drawLine(getShiftX(), 0, getShiftX() - getRatio(), getRatio());
	g.drawLine(getShiftX(), 0, getShiftX() + getRatio(), getRatio());
	g.drawLine((int) getSize().getWidth(), getShiftY(), (int) getSize()
		.getWidth() - getRatio(), getShiftY() - getRatio());
	g.drawLine((int) getSize().getWidth(), getShiftY(), (int) getSize()
		.getWidth() - getRatio(), getShiftY() + getRatio());
    }

    void signAxes(Graphics g) {
	g.drawString(nameAxeY, getShiftX() + getRatio() * 2, getRatio() * 2);
	g.drawString(nameAxeX, (int) getSize().getWidth() - getRatio() * 2,
		getShiftY() + getRatio() * 2);
    }

    void buildSchedule(Graphics g) {
	for (int i = 0; i < (list.size() - 1); i++) {
	    double[] xy1 = list.get(i);
	    double[] xy2 = list.get(i + 1);
	    int x1 = (int) (xy1[0] * getValueOfDivisionX() + getShiftX());
	    int y1 = (int) (getShiftY() - xy1[1] * getValueOfDivisionY());
	    int x2 = (int) (xy2[0] * getValueOfDivisionX() + getShiftX());
	    int y2 = (int) (getShiftY() - xy2[1] * getValueOfDivisionY());
	    if ((x1 < 0 || x1 > getSize().getWidth() || y1 < 0 || y1 > getSize()
		    .getHeight())
		    && (x2 < 0 || x2 > getSize().getWidth() || y2 < 0 || y2 > getSize()
			    .getHeight())) {
		continue;
	    }
	    g.drawLine(x1, y1, x2, y2);
	}
    }

    public double getFrequencyDivisionX() {
	return 60 / getValueOfDivisionX();
    }

    public double getFrequencyDivisionY() {
	return 60 / getValueOfDivisionY();
    }

    void paintDivision(Graphics g) {
	for (double i = getFrequencyDivisionX(); i <= (getShiftX() / getValueOfDivisionX()); i = i
		+ getFrequencyDivisionX()) {

	    g.drawLine(getShiftX() - (int) (getValueOfDivisionX() * i),
		    getShiftY() + getRatio(), getShiftX()
			    - (int) (getValueOfDivisionX() * i), getShiftY()
			    - getRatio());
	    g.drawString((new Double(roundResult((-i), 3))).toString(),
		    getShiftX() - (int) (getValueOfDivisionX() * i),
		    getShiftY() + 3 * getRatio());

	}
	for (double i = getFrequencyDivisionX(); i <= (Math.abs((getShiftX()
		- getSize().getWidth() + getRatio() * 2)
		/ getValueOfDivisionX())); i = i + getFrequencyDivisionX()) {
	    g.drawLine(getShiftX() + (int) (getValueOfDivisionX() * i),
		    getShiftY() + getRatio(), getShiftX()
			    + (int) (getValueOfDivisionX() * i), getShiftY()
			    - getRatio());
	    g.drawString((new Double(roundResult((i), 3))).toString(),
		    getShiftX() + (int) (getValueOfDivisionX() * i),
		    getShiftY() + 3 * getRatio());

	}
	for (double i = getFrequencyDivisionY(); i <= ((getSize().getHeight() - getShiftY()) / valueOfDivisionY); i = i
		+ getFrequencyDivisionY()) {
	    g.drawLine(getShiftX() - getRatio(), getShiftY()
		    + (int) (getValueOfDivisionY() * i), getShiftX()
		    + getRatio(), getShiftY()
		    + (int) (getValueOfDivisionY() * i));
	    g.drawString(((new Double(roundResult((-i), 3)))).toString(),
		    getShiftX() + 2 * getRatio(), getShiftY()
			    + (int) (getValueOfDivisionY() * i));

	}
	for (double i = getFrequencyDivisionY(); i <= ((getShiftY() - getRatio() * 2) / valueOfDivisionY); i = i
		+ getFrequencyDivisionY()) {
	    g.drawLine(getShiftX() - getRatio(), getShiftY()
		    - (int) (getValueOfDivisionY() * i), getShiftX()
		    + getRatio(), getShiftY()
		    - (int) (getValueOfDivisionY() * i));
	    g.drawString(((new Double(roundResult((i), 3)))).toString(),
		    getShiftX() + 2 * getRatio(), getShiftY()
			    - (int) (getValueOfDivisionY() * i));

	}
    }

    private double roundResult(double aValue, int precise) {
	BigDecimal decimal = new BigDecimal(aValue);
	decimal = decimal.setScale(precise, BigDecimal.ROUND_DOWN);
	double result = decimal.doubleValue();
	return result;
    }

    @Override
    public void update() {
	if (model.getGraph() != null) {
	    list = model.getGraph();
	}
	repaint();
    }

}